#!/bin/env python2
"""
KBBTray Big Brother KDE System Tray
Copyright (C) 2003 Russell Valentine 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import sys
import string
from string import strip
import urllib
import re
from kfile import KFileDialog
from kdeui import KMainWindow, KSystemTray, KPopupMenu, KDialogBase, KIntNumInput, KLineEdit, KAboutDialog, KAction
from kdecore import KApplication, KIconLoader, KIcon, KAboutData, KCmdLineArgs, i18n, KAudioPlayer
from HTMLParser import HTMLParser
from qt import QPushButton, QLabel, QCursor, SIGNAL, SLOT, QThread, QPixmap, QColor, Qt, QObject, QHBox, QMovie, QToolTip, QWidget, QGridLayout, QGroupBox, QString


class KBBTrayChecker(QThread):
    def __init__(self, parent, *args):
        QThread.__init__(self)
        self.parentTray = parent
        self.doStop = None
        self.refreshConfig()
    def refreshConfig(self):
        config = self.parentTray.parentApp.config()
        config.setGroup("Options")
        self.urlGet = str(config.readEntry("bbURL", None))
        self.pollTime = config.readNumEntry("pollw", 30)
        self.counter = self.pollTime*60
    def stop(self):
        self.doStop = 1
    def run(self):
        status = []
        while not self.doStop:
            if self.counter < self.pollTime*60:
                self.counter = self.counter+1
                self.sleep(1)
            else:
                self.counter = 0
                #Do check
                color=None
                try:
                    if self.urlGet == None:
                        color="unknown"
                    else:
                        f = urllib.urlopen(self.urlGet)
                        page = f.read()
                        f.close
                        parser = MyHTMLParser()
                        parser.feed(page)
                        parser.close()
                        color = parser.get_color()
                        status = parser.get_status()
                except:
                    print "Problem connecting or parsing:"+self.urlGet
                    color="unknown"
                    raise
                if color == "green":
                    self.parentTray.setStatus("ok", status)
                elif color == "yellow":
                    self.parentTray.setStatus("attention", status)
                elif color == "red":
                    self.parentTray.setStatus("trouble", status)
                elif color == "purple":
                    self.parentTray.setStatus("no_report", status)
                elif color == "clear":
                    self.parentTray.setStatus("unavailable", status)
                elif color == "blue":
                    self.parentTray.setStatus("offline", status)
                else:
                    self.parentTray.setStatus("unknown", status)


#  To parse the bb page for the color of the title #
#  and to parse for what is wrong.  Assumes there  #
#  are not tables inside of tables.                #
class MyHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.title_seen = 0
        self.group_block_seen = 0 
        self.table_row_seen = 0
        self.font_seen = 0
        self.data = ""
        self.table_data = ""
        self.font_data = ""
        self.status = []
        self.title_color = ""

    def handle_starttag(self, tag, attrs):
        #Look for the <TITLE> tag
        if tag == "title":
            self.title_seen = 1
        #Look for the <TR> tag
        if tag == "tr" and self.group_block_seen: 
            self.table_row_seen = 1
        #Look for the <FONT> tag
        if tag == "font" and self.group_block_seen: 
            self.font_seen = 1
        #Look for the <TABLE> tag
        if tag == "table":
            for attribute, value in attrs:        
                if attribute == "summary" and value == "Group Block": 
                    self.group_block_seen = 1        
        #Look for the <IMG> tag
        if tag == "img" and self.table_row_seen:
            for attribute, value in attrs:        
                if attribute == "alt": 
                    color = value.split(":")[1]
                    if color != "green":
                        self.status.append(self.font_data + ":" + value)
    def handle_endtag(self, tag):
        #Close the Title
        if tag == "title":
            self.title_color = self.data.split(" :")[0]
            self.title_seen = 0
            self.data = ""
        #If we are in a table row and inside a block then reset the font data to ""
        if tag == "tr" and self.group_block_seen: 
            self.table_row_seen = 0
            self.font_data = ""
            #Hopefully at the end of a group block
        if tag == "table":
            self.group_block_seen = 0
            #No more font entry
        if tag == "font" and self.group_block_seen: 
            self.font_seen = 0
    def handle_data(self,data):
        if data:
            if self.title_seen:
                self.data = self.data + data
            if self.font_seen:
                self.font_data = self.font_data + data
    def get_color(self):
        return self.title_color
    def get_status(self):
        return self.status

class KBBConfigureDialog(KDialogBase):
    def __init__(self, parent):
        KDialogBase.__init__(self, parent, None, 1, "Configuration", KDialogBase.Ok|KDialogBase.Cancel, KDialogBase.Ok, 1)
        self.parentTray = parent
        self.appConfig = self.parentTray.parentApp.config()
        #TODO: Add validators
        page = self.makeVBoxMainWidget()
        exURLLabel = QLabel("(ex. http://user:password@bbserver.com:80/index.html)", page)
        urlbox = QHBox(page)
        bbURLLable = QLabel("BB URL:", urlbox)
        self.bbURL = KLineEdit(self.appConfig.readEntry("bbURL", ""), urlbox)
        pollbox = QHBox(page)
        pollLabel1 = QLabel("Poll server every:", pollbox)
        self.pollw = KIntNumInput(self.appConfig.readNumEntry("pollw", 2), pollbox)
        self.pollw.setRange(1, 30)
        pollLabel2 = QLabel("Minutes", pollbox)
        #audio sounds
        audiobox = QGroupBox(4, Qt.Horizontal, i18n("Audio Sounds"), page)
        iconLoader = KIconLoader()
        #green
        greenLabel = QLabel("Green sound:", audiobox)
        self.greenSoundFile = KLineEdit(self.appConfig.readEntry("greenSound", ""), audiobox)
        greenButton = QPushButton(audiobox)
        greenButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(greenButton, SIGNAL("released()"), self.pickFileGreen)
        gtestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(gtestButton, SIGNAL("released()"), self.testGreen)
        #yellow
        yelloLabel = QLabel("Yellow sound:", audiobox)
        self.yellowSoundFile = KLineEdit(self.appConfig.readEntry("yellowSound", ""), audiobox)
        yellowButton = QPushButton(audiobox)
        yellowButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(yellowButton, SIGNAL("released()"), self.pickFileYellow)
        ytestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ytestButton, SIGNAL("released()"), self.testYellow)
        #red
        redLabel = QLabel("Red sound:", audiobox)
        self.redSoundFile = KLineEdit(self.appConfig.readEntry("redSound", ""), audiobox)
        redButton = QPushButton(audiobox)
        redButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(redButton, SIGNAL("released()"), self.pickFileRed)
        rtestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(rtestButton, SIGNAL("released()"), self.testRed)
        #purple
        purpleLabel = QLabel("Purple sound:", audiobox)
        self.purpleSoundFile = KLineEdit(self.appConfig.readEntry("purpleSound", ""), audiobox)
        purpleButton = QPushButton(audiobox)
        purpleButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(purpleButton, SIGNAL("released()"), self.pickFilePurple)
        ptestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ptestButton, SIGNAL("released()"), self.testPurple)
        #clear
        clearLabel = QLabel("Clear sound:", audiobox)
        self.clearSoundFile = KLineEdit(self.appConfig.readEntry("clearSound", ""), audiobox)
        clearButton = QPushButton(audiobox)
        clearButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(clearButton, SIGNAL("released()"), self.pickFileClear)
        ctestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ctestButton, SIGNAL("released()"), self.testClear)
        #blue
        blueLabel = QLabel("Blue sound:", audiobox)
        self.blueSoundFile = KLineEdit(self.appConfig.readEntry("blueSound", ""), audiobox)
        blueButton = QPushButton(audiobox)
        blueButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(blueButton, SIGNAL("released()"), self.pickFileBlue)
        btestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(btestButton, SIGNAL("released()"), self.testBlue)
    def testGreen(self):
        KAudioPlayer.play(self.greenSoundFile.text())
    def testYellow(self):
        KAudioPlayer.play(self.yellowSoundFile.text())
    def testRed(self):
        KAudioPlayer.play(self.redSoundFile.text())
    def testPurple(self):
        KAudioPlayer.play(self.purpleSoundFile.text())
    def testClear(self):
        KAudioPlayer.play(self.clearSoundFile.text())
    def testBlue(self):
        KAudioPlayer.play(self.blueSoundFile.text())
    def pickFile(self, color):
        url=KFileDialog.getOpenURL(None, "*.wav|"+str(i18n("Wav files"))+"\n*.*|"+str(i18n("All files")), self, "Choose File")
        if url.isEmpty() == 0:
            if color == "green":
                self.greenSoundFile.setText(url.path())
            elif color == "yellow":
                self.yellowSoundFile.setText(url.path())
            elif color == "red":
                self.redSoundFile.setText(url.path())
            elif color == "purple":
                self.purpleSoundFile.setText(url.path())
            elif color == "clear":
                self.clearSoundFile.setText(url.path())
            elif color == "blue":
                self.blueSoundFile.setText(url.path())                
    def pickFileGreen(self):
        self.pickFile("green")
    def pickFileYellow(self):
        self.pickFile("yellow")
    def pickFileRed(self):
        self.pickFile("red")
    def pickFilePurple(self):
        self.pickFile("purple")
    def pickFileClear(self):
        self.pickFile("clear")
    def pickFileBlue(self):
        self.pickFile("blue")
    def slotOk(self):
        self.appConfig.setGroup("Options")
        self.appConfig.writeEntry("bbURL", self.bbURL.text())
        self.appConfig.writeEntry("pollw", self.pollw.value())
        self.appConfig.writeEntry("greenSound", self.greenSoundFile.text())
        self.appConfig.writeEntry("yellowSound", self.yellowSoundFile.text())
        self.appConfig.writeEntry("redSound", self.redSoundFile.text())
        self.appConfig.writeEntry("purpleSound", self.purpleSoundFile.text())
        self.appConfig.writeEntry("clearSound", self.clearSoundFile.text())
        self.appConfig.writeEntry("blueSound", self.blueSoundFile.text())
        self.appConfig.sync()
        self.parentTray.updateAudio()
        self.parentTray.checker.refreshConfig()
        self.accept()

class KBBTrayWindow(KMainWindow):
    def __init__(self):
        KMainWindow.__init__(self)
        
        self.view = View(self)
        self.setCentralWidget(self.view)
        
    def update(self, dstatus):
        self.view.update(dstatus)
        
    def queryClose(self):
        self.hide()
        return 0
        
class View(QWidget):
    def __init__(self, window):
        QWidget.__init__(self, window)
        self.grid = QGridLayout(self, 2, 1, 10)
        self.grid.addWidget(QLabel("Non-Green Status:", self), 0,0)
        self.label = QLabel("No data availible yet", self)
        self.grid.addWidget(self.label, 1,0)
        
    def update(self, dstatus):
        newerLabel = ""
        if len(dstatus) == 0:
            self.label.setText("All Green")
        else:
            for value in dstatus:
                newerLabel += value+"\n"
            self.label.setText(newerLabel)

class KBBSystemTray(KSystemTray):
    def __init__(self, window, app):
        KSystemTray.__init__(self, window)
        self.parentApp = app
        self.window = window
        #Setup popup menu
        #self.contextMenu().clear()
        iconLoader = KIconLoader()
        self.contextMenu().insertItem(iconLoader.loadIconSet("configure", KIcon.Small), "&Configure KBBTray", self.openConfigDialog)
        #self.contextMenu().insertItem("&About KBBTray", self.doAboutDialog)
        QObject.connect(self, SIGNAL("quitSelected()"), self.parentApp, SLOT("quit()"))
        
        #Create all status icons
        #TODO: Use a movie for animated icons
        #unkown is couldn't connect, or in process
        self.unknownPixmap = iconLoader.loadIcon("connect_creating", KIcon.Small)
        self.okPixmap = QPixmap("/usr/share/kbbtray/green.png")
        self.attentionMovie = QMovie("/usr/share/kbbtray/yellow.mng")           
        self.troubleMovie = QMovie("/usr/share/kbbtray/red.mng")
        self.no_reportMovie = QMovie("/usr/share/kbbtray/purple.mng")
        self.unavailablePixmap = QPixmap("/usr/share/kbbtray/clear.png")
        self.offlinePixmap = QPixmap("/usr/share/kbbtray/blue.png")
        
        #Audios
        self.greenAudio = None
        self.yellowAudio = None
        self.redAudio = None
        self.purpleAudio = None
        self.clearAudio = None
        self.blueAudio = None
        self.updateAudio()
        
        self.status = None
        self.dStatus = []
        self.setStatus("unknown", self.dStatus)
        #Last after icons made start thread checker
        self.checker=KBBTrayChecker(self)
        self.checker.start()
    def updateAudio(self):
        appConfig = self.parentApp.config()
        appConfig.setGroup("Options")
        self.greenAudio = KAudioPlayer(QString(appConfig.readEntry("greenSound", "")), self)
        self.yellowAudio = KAudioPlayer(QString(appConfig.readEntry("yellowSound", "")), self)
        self.redAudio = KAudioPlayer(QString(appConfig.readEntry("redSound", "")), self)
        self.purpleAudio = KAudioPlayer(QString(appConfig.readEntry("purpleSound", "")), self)
        self.clearAudio = KAudioPlayer(QString(appConfig.readEntry("clearSound", "")), self)
        self.blueAudio = KAudioPlayer(QString(appConfig.readEntry("blueSound", "")), self)
    
    def __del__(self):
        if self.checker.running() : self.checker.stop()
    def setStatus(self, theStatus, detailed_status):
        tooltip = ""
        update = 0
        colorUpdate = 0
        if self.status != theStatus:
            update = 1
            colorUpdate = 1
        else:
            if len(detailed_status) != len(self.dStatus):
                update = 1
            else:
                for i in range(len(detailed_status)):
                    if detailed_status[i] != self.dStatus[i]:
                        update = 1
                        break
        if update != 0:
            self.status = theStatus
            self.dStatus = detailed_status
            self.parentApp.lock()
            QToolTip.add(self, "KBBTray")
            if colorUpdate == 1:
                #Change icon here
                if self.status == "unknown":
                    self.setPixmap(self.unknownPixmap)
                elif self.status == "ok":
                    self.setPixmap(self.okPixmap)
                elif self.status == "attention":
                    self.setMovie(self.attentionMovie)
                elif self.status == "trouble":
                    self.setMovie(self.troubleMovie)
                elif self.status == "no_report":
                    self.setMovie(self.no_reportMovie)
                elif self.status == "unavailable":
                    self.setPixmap(self.unavailablePixmap)
                elif self.status == "offline":
                    self.setPixmap(self.offlinePixmap)                
            for i in range(len(self.dStatus)):
                tooltip = tooltip + self.dStatus[i] 
                if i != len(self.dStatus)-1:
                    tooltip = tooltip+"\n"
            if tooltip == "":
                tooltip = "All Green"
            QToolTip.add(self, tooltip)
            self.window.update(self.dStatus)
            if colorUpdate == 1:
                #Play color sound
                if self.status == "ok":
                    if self.greenAudio != None:
                        self.greenAudio.play()
                elif self.status == "attention":
                    if self.yellowAudio != None:
                        self.yellowAudio.play()
                elif self.status == "trouble":
                    if self.redAudio != None:
                        self.redAudio.play()
                elif self.status == "no_report":
                    if self.purpleAudio != None:
                        self.purpleAudio.play()
                elif self.status == "unavailable":
                    if self.clearAudio != None:
                        self.clearAudio.play()
                elif self.status == "offline":
                    if self.blueAudio != None:
                        self.blueAudio.play()
            self.parentApp.unlock()
    def openConfigDialog(self):
        configDialog = KBBConfigureDialog(self)
        configDialog.show()
    def doAboutDialog(self):
        theAboutDialog = KAboutDialog(self)
        aboutData = self.parentApp.aboutData()
        theAboutDialog.show()

def main():
    aboutData = KAboutData("kbbtray", "KBBTray", "0.1", "A KDE System Tray Big Brother checker", KAboutData.License_GPL, "(c) 2003 Russell E. Valentine", None, "http://coldstonelabs.org", "russ@coldstonelabs.org");
    aboutData.addAuthor("Russell Valentine", "Main Programmer", "russ@coldstonelabs.org", "http://users.coldstonelabs.org/russ/");
    aboutData.addAuthor("Michael Garrett", "Programmer/Idea");
    KCmdLineArgs.init(sys.argv, aboutData)
    app=KApplication()
    win=KBBTrayWindow()
    tray=KBBSystemTray(win, app)
    tray.show()
    app.exec_loop()

if __name__=="__main__":
    main()
